Class: Interaktor::Error::MissingExplicitSuccessError
- Inherits:
-
Base
- Object
- Base
- Interaktor::Error::MissingExplicitSuccessError
- Defined in:
- lib/interaktor/error/missing_explicit_success_error.rb
Instance Method Summary collapse
Instance Method Details
#message ⇒ Object
2 3 4 5 6 7 8 9 |
# File 'lib/interaktor/error/missing_explicit_success_error.rb', line 2 def <<~MSG.gsub(/\s+/, " ") #{interaktor} interaktor execution finished successfully, but the interaktor definition includes a `success` attribute definition, and as a result the interaktor must call the `success!` method with the appropriate attributes. MSG end |